<?php
namespace Zeedhi\DataMining\Common\Service\DataCollector;

class SocketClient {

    const NSP        = 'ZmartDataCollector';
    const SERVER_KEY = 'you can create!';
    const PING_ROOM  = 'PING_ROOM';

    // Events
    const CREATE_NSP         = 'createNamespace';
    const CREATE_ROOM        = 'createRoom';
    const JOIN_ROOM          = 'joinRoom';
    const EMIT               = 'emit';
    const PING               = 'pingConnection';
    const START_IMPORTATION  = 'startImportation';
    const FINISH_IMPORTATION = 'finishImportation';
    const NOTIFY_ERROR       = 'onError';
    const UPDATE_IMPORT_PROCESS = 'updateImportProcess';
    const GET_CREATOR_DATASOURCE = 'getCreatorDatasource';

    protected $socketServerAddress;
    protected $socketClient;

    public function __construct($socketServerAddress) {
        $this->socketServerAddress = $socketServerAddress;
    }

    protected function connectToWebSocketServer() {
        $this->socketClient = new \WebSocket\Client($this->socketServerAddress);
        $this->connectToZmartNamespace();
    }

    public function connect() {
        $this->connectToWebSocketServer();
    }

    public function ping() {
        $this->connectToWebSocketServer();
        $this->send($this->buildBasePayload(self::EMIT, self::PING_ROOM, array('key' => self::SERVER_KEY, 'event' => self::PING)));
    }

    public function startImportation($organization, $process, $query, $queryKeys, $pageSize, $dbUrl, $userName, $password,
                                     $scriptBefore, $scriptAfter, $execAfterIfImportError, $execAfterIfBeforeScriptError,
                                     $clientRoom, $dialect, $driverClass, $usernameSsl, $passwordSsl, $dbUrlSsl) {
        $this->joinRoom($clientRoom);

        $data = array(
            'organizationId'               => $organization,
            'event'                        => self::START_IMPORTATION,
            'process'                      => $process,
            'query'                        => $query,
            'queryKeys'                    => isset($queryKeys) ? $queryKeys : array(),
            'pageSize'                     => $pageSize,
            'dbUrl'                        => $dbUrl,
            'userName'                     => $userName,
            'password'                     => $password,
            'dbUrlSsl'                     => $dbUrlSsl,
            'userNameSsl'                  => $usernameSsl,
            'passwordSsl'                  => $passwordSsl,
            'scriptBefore'                 => $scriptBefore,
            'scriptAfter'                  => $scriptAfter,
            'execAfterIfImportError'       => $execAfterIfImportError,
            'execAfterIfBeforeScriptError' => $execAfterIfBeforeScriptError,
            'driverClass'                  => $driverClass,
            'dialect'                      => $dialect
        );

        $this->send($this->buildBasePayload(self::EMIT, $clientRoom, $data));
    }

    public function notifyImportationEnd($organization, $dataSource, $updateIndicators) {
        $this->connectToWebSocketServer();
        $this->joinRoom($organization);

        $data = array(
            'event'      => self::FINISH_IMPORTATION,
            'organizationId' => $organization,
            'dataSource' => $dataSource,
            'updateIndicators' => $updateIndicators
        );

        $message = $this->buildBasePayload(self::EMIT, $organization, $data);
        $this->send($message);
    }

    public function notifyErrorOfEmail($processHistory, $creatorId, $dataSourceName, $message){
        $organization = $processHistory['organizationId'];

        $this->connectToWebSocketServer();
        $this->joinRoom($organization);

        $data = array(
                'event'                 => self::GET_CREATOR_DATASOURCE,
                'dataSourceName'        => $dataSourceName,
                'organizationId'          => $organization,
                'idCreatorDataSource'   => $creatorId,
                'message'               => $message
            );

        $message = $this->buildBasePayload(self::EMIT, $organization, $data);
        $this->send($message);
    }

    protected function formatDate($date) {
        if ($date instanceof \MongoDB\BSON\UTCDateTime) {
            $dateClone = new \DateTime();
            $dateClone->setTimestamp($date->toDateTime()->getTimestamp());
            $formattedDate = $dateClone->format('d/m/Y H:i');
        } else {
            $formattedDate = $date;
        }

        return $formattedDate;
    }

    public function notifyImportUpdate($organization, $processHistory){
        $this->connectToWebSocketServer();
        $this->joinRoom($organization);

        $processHistory['createDate'] = $this->formatDate($processHistory['createDate']);
        $processHistory['endDate'] = $this->formatDate($processHistory['endDate']);

        $data = array(
            'event'        => self::UPDATE_IMPORT_PROCESS,
            'organizationId' => $organization,
            'process'      => $processHistory
        );

        $message = $this->buildBasePayload(self::EMIT, $organization, $data);
        $this->send($message);
    }

    public function notifyError($organization, $process) {
        $this->joinRoom($organization);

        $data = array(
            'event'   => self::NOTIFY_ERROR,
            'process' => $process
        );
        $this->send($this->buildBasePayload(self::EMIT, $organization, $data));
    }

    protected function send(array $payload) {
        $jsonToSend = json_encode($payload);
        $this->socketClient->send($jsonToSend);
    }

    protected function connectToZmartNamespace() {
        $this->send($this->buildBasePayload(self::CREATE_NSP, null, array('key' => self::SERVER_KEY)));
    }

    protected function joinRoom($room) {
        $this->createRoom($room);
        $this->send($this->buildBasePayload(self::JOIN_ROOM, $room));
    }

    protected function createRoom($room) {
        $this->send($this->buildBasePayload(self::CREATE_ROOM, $room));
    }

    protected function buildBasePayload($event, $room = null, $data = array()) {
        return array(
            'event'     => $event,
            'namespace' => self::NSP,
            'room'      => $room,
            'data'      => $data
        );
    }

}